// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "dawnlibs.hpp"

namespace Dawn {
  enum CardSuit {
    CARD_CLUBS = 0,
    CARD_DIAMONDS = 1,
    CARD_HEARTS = 2,
    CARD_SPADES = 3
  };

  enum CardValue {
    CARD_TWO = 0,
    CARD_THREE = 1,
    CARD_FOUR = 2,
    CARD_FIVE = 3,
    CARD_SIX = 4,
    CARD_SEVEN = 5,
    CARD_EIGHT = 6,
    CARD_NINE = 7,
    CARD_TEN = 8,
    CARD_JACK = 9,
    CARD_QUEEN = 10,
    CARD_KING = 11,
    CARD_ACE = 12
  };

  /** Count of cards in each suit */
  #define CARD_COUNT_PER_SUIT 13

  /** Count of suits */
  #define CARD_SUIT_COUNT 4

  /** Standard Card Deck Size */
  #define CARD_DECK_SIZE CARD_COUNT_PER_SUIT*CARD_SUIT_COUNT

  struct Card {
    public:
      uint8_t cardValue;

      static void fillDeck(std::vector<struct Card> *deck); 
      static void shuffle(std::vector<struct Card> *deck);
      static bool_t contains(std::vector<struct Card> *deck, struct Card);
      static std::vector<struct Card> countPairs(
        std::vector<struct Card> *deck,
        enum CardValue value
      );

      /**
       * Sort a hand of cards. Cards are ordered in descending weight, aces are
       * high. Cards will be grouped by their suits, e.g. CARD_CLUBS_TWO will
       * appear before CARD_DIAMONDS_KING.
       * 
       * @param deck Hand of cards to sort.
       */
      static void sort(std::vector<struct Card> *deck);

      Card(CardSuit suit, CardValue num) :
        cardValue((suit * CARD_COUNT_PER_SUIT) + num)
      {
        assertTrue(suit < CARD_SUIT_COUNT);
        assertTrue(num < CARD_COUNT_PER_SUIT);
      }

      Card(uint8_t cv) : cardValue(cv) {
        assertTrue(cv < CARD_DECK_SIZE);
      }

      CardValue getValue() {
        return (CardValue)(cardValue % CARD_COUNT_PER_SUIT);
      }

      CardSuit getSuit() {
        return (CardSuit)(cardValue / CARD_COUNT_PER_SUIT);
      }
  };

  bool_t cardSorter(struct Card left, struct Card right);
}