// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "scene/Scene.hpp"
#include "game/DawnGame.hpp"
#include "scene/components/Components.hpp"
#include "ui/PokerGameTextbox.hpp"
#include "visualnovel/VisualNovelManager.hpp"
#include "visualnovel/events/VisualNovelTextboxEvent.hpp"
#include "poker/PokerGame.hpp"
#include "poker/visualnovel/PokerNewGameEvent.hpp"
#include "poker/visualnovel/PokerNewRoundEvent.hpp"
#include "poker/visualnovel/PokerTakeBlindsEvent.hpp"
#include "poker/visualnovel/PokerDealEvent.hpp"
#include "poker/visualnovel/PokerTurnEvent.hpp"
#include "poker/visualnovel/PokerDetermineBetterEvent.hpp"

namespace Dawn {
  class TestScene {
    public:
      static Scene * create(DawnGame *game) {
        Scene *scene = new Scene(game);

        // Camera
        auto camera = Camera::create(scene);
        camera->transform->lookAt(glm::vec3(50, 50, 50), glm::vec3(0, 0, 0));
        
        // UI
        auto canvas = UICanvas::createCanvas(scene);
        auto textbox = PokerGameTextbox::create(canvas);

        // VN Manager
        auto vnManagerItem = scene->createSceneItem();
        auto vnManager = vnManagerItem->addComponent<VisualNovelManager>();

        // Poker Test
        auto pokerGameItem = scene->createSceneItem();
        auto pokerGame = pokerGameItem->addComponent<PokerGame>();

        for(int32_t i = 0; i < 4; i++) {
          auto pokerPlayerInstance = scene->createSceneItem();
          auto pokerPlayer = pokerPlayerInstance->addComponent<PokerPlayer>();
        }

        auto betting = vnManager
          ->setEvent(new VisualNovelTextboxEvent(vnManager, "Starting Game"))
          ->then(new PokerNewGameEvent(vnManager))
          ->then(new VisualNovelTextboxEvent(vnManager, "Game Started"))
          ->then(new PokerNewRoundEvent(vnManager))
          ->then(new VisualNovelTextboxEvent(vnManager, "Round Started"))
          ->then(new PokerTakeBlindsEvent(vnManager))
          ->then(new VisualNovelTextboxEvent(vnManager, "Blinds Taken"))
          ->then(new PokerDealEvent(vnManager))
          ->then(new VisualNovelTextboxEvent(vnManager, "Cards Dealt"))
          ->then(new PokerDetermineBetterEvent(vnManager))
        ;
        betting
          ->whenEveryoneFolded(new VisualNovelTextboxEvent(vnManager, "Everyone Folded"))
        ;
        betting
          ->whenBettingFinished(new VisualNovelTextboxEvent(vnManager, "Betting Finished"))
        ;
        betting
          ->whenTurn(new VisualNovelTextboxEvent(vnManager, "Turn Time"))
        ;
        betting
          ->whenAiBet(new VisualNovelTextboxEvent(vnManager, "AI Bet"))
        ;
        betting
          ->whenHumanBet(new VisualNovelTextboxEvent(vnManager, "Human Bet"))
        ;

        return scene;
      }
  };
}